    -- ResolverAD2S1200DecA7  - Resolver connected to AD2S1200 decoder with CmodA7 board
    -- Copyright (C) 2021 VCC
    -- release date: 31 Oct 2021
    -- author: VCC

    -- This file is part of ResolverAD2S1200DecA7 project.

    -- Permission is hereby granted, free of charge, to any person obtaining a copy
    -- of this software and associated documentation files (the "Software"),
    -- to deal in the Software without restriction, including without limitation
    -- the rights to use, copy, modify, merge, publish, distribute, sublicense,
    -- and/or sell copies of the Software, and to permit persons to whom the
    -- Software is furnished to do so, subject to the following conditions:

    -- The above copyright notice and this permission notice shall be included
    -- in all copies or substantial portions of the Software.

    -- THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
    -- EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
    -- MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
    -- IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM,
    -- DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,
    -- TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE
    -- OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.


----------------------------------------------------------------------------------
-- Company: 
-- Engineer: VCC
-- 
-- Create Date:    23:49:42 10/17/2021 
-- Design Name: 
-- Module Name:    AD2S1200Interface - Behavioral 
-- Project Name: 
-- Target Devices: 
-- Tool versions: 
-- Description: 
--
-- Dependencies: 
--
-- Revision: 
-- Revision 0.01 - File Created
-- Additional Comments: 
--
----------------------------------------------------------------------------------
library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;

-- Uncomment the following library declaration if using
-- arithmetic functions with Signed or Unsigned values
--use IEEE.NUMERIC_STD.ALL;

-- Uncomment the following library declaration if instantiating
-- any Xilinx primitives in this code.
--library UNISIM;
--use UNISIM.VComponents.all;

entity AD2S1200Interface is
    Port (
      Clk100MHz: in STD_LOGIC;
      WantsToReadVelocity: in STD_LOGIC; -- when set to '1', the AD2S is asked about velocity
      
      AD2SData: in STD_LOGIC_VECTOR(11 downto 0);
      SAMPLE: out STD_LOGIC;
      RD: out STD_LOGIC;
      CS: out STD_LOGIC;
      RDVELOut: out STD_LOGIC; --Read velocity/position  - '0' = velocity, '1' = position

      Position: out STD_LOGIC_VECTOR(11 downto 0);
      Velocity: out STD_LOGIC_VECTOR(11 downto 0)
    );
end AD2S1200Interface;

architecture Behavioral of AD2S1200Interface is

  signal MainCounter: STD_LOGIC_VECTOR(7 downto 0) := (others => '0');
  
  signal PosReg: STD_LOGIC_VECTOR(11 downto 0) := (others => '0');
  signal VelReg: STD_LOGIC_VECTOR(11 downto 0) := (others => '0');
    
  signal OldAD2SData: STD_LOGIC_VECTOR(11 downto 0);
begin

  --------------------------------------------------------------- all timings must be recalculated for a clock, which is 5 times faster  (100 vs 20)
  --------------------------------------------------------------- translate comments

  process(Clk100MHz)
  begin
    if Clk100MHz'event and Clk100MHz = '1' then
      MainCounter <= MainCounter + 1;
      OldAD2SData <= AD2SData;
      
      ------------------------------------------------------------------------------------------------------------------------------------------------------------------------
      ---------------------------- The following timings are experimental, based on a particular board with its own delays, introduced by series resistors / signal stability.
      ---------------------------- Smaller timings can be set if the board supports it.
      ------------------------------------------------------------------------------------------------------------------------------------------------------------------------
      
      if MainCounter = 1 then 
        SAMPLE <= '0';
        CS <= '1';
        RD <= '1';
        RDVELOut <= '1'; -- '1' means read position from AD2S
      end if;

      --t1 is the length of SAMPLE on '0'. Must be at least 264ns.
      if MainCounter = 31 then --30 * 10ns = 300ns.  (31 = 30 + 1, because it starts at 1)
        SAMPLE <= '1';
      end if;
      
      --t2 starts at MainCounter = 1. Must be at least 752.42ns
      if MainCounter = 81 then --80 * 10ns = 800ns (81 = 80 + 1)
        CS <= '0';
        RD <= '0'; --RD was already '1' at MainCounter = 1
      end if;
      
      --t6 starts at MainCounter = 81. In at least 12ns, data should be available.
      if MainCounter = 88 then -- (87 - 81) * 10ns = 60ns   --with 86, the reading may still be affected by noise on some positions
        PosReg <= OldAD2SData; --use the registered version of AD2SData, to prevent a bit of metastability
      end if;                  -- the comparison is against 88, not 87, because of using OldAD2SData instead of plain AD2SData
      
      if WantsToReadVelocity = '1' then
        --the duration of data availability depends on when RD goes back to '1':
        if MainCounter = 92 then -- (92 - 87) * 10ns = 50ns
          RDVELOut <= '0'; -- '0' means read velocity from AD2S
          CS <= '1';
          RD <= '1';
        end if;
        
        --t4 (at least 5ns) is the required time before RD/CS going low
        if MainCounter = 95 then -- (95 - 92) * 10ns = 30ns
          CS <= '0';
          RD <= '0';
        end if;
        
        if MainCounter = 102 then -- (101 - 95) * 10ns = 60ns
          VelReg <= OldAD2SData;  -- use the registered version of AD2SData, to prevent a bit of metastability
        end if;
        
        if MainCounter = 107 then -- (107 - 101) * 10ns = 60ns
          RDVELOut <= '1';
          MainCounter <= (others => '0');
        end if;
      else --WantsToReadVelocity = '0'
        if MainCounter = 92 then -- reset the counter if velocity is not read
          RDVELOut <= '1';
          MainCounter <= (others => '0');
        end if;
      end if; --WantsToReadVelocity = '1'
      
      
      
    end if; --'event
  end process;
  
  
  Position <= PosReg;
  Velocity <= VelReg;

end Behavioral;

