    -- ResolverAD2S1200DecA7  - Resolver connected to AD2S1200 decoder with CmodA7 board
    -- Copyright (C) 2021 VCC
    -- release date: 31 Oct 2021
    -- author: VCC

    -- This file is part of ResolverAD2S1200DecA7 project.

    -- Permission is hereby granted, free of charge, to any person obtaining a copy
    -- of this software and associated documentation files (the "Software"),
    -- to deal in the Software without restriction, including without limitation
    -- the rights to use, copy, modify, merge, publish, distribute, sublicense,
    -- and/or sell copies of the Software, and to permit persons to whom the
    -- Software is furnished to do so, subject to the following conditions:

    -- The above copyright notice and this permission notice shall be included
    -- in all copies or substantial portions of the Software.

    -- THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
    -- EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
    -- MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
    -- IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM,
    -- DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,
    -- TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE
    -- OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.


----------------------------------------------------------------------------------
-- Company: 
-- Engineer: VCC
-- 
-- Create Date:    12:18:57 10/17/2021 
-- Design Name: 
-- Module Name:    AD2SErrorHandling - Behavioral 
-- Project Name: 
-- Target Devices: 
-- Tool versions: 
-- Description: 
--
-- Dependencies: 
--
-- Revision: 
-- Revision 0.01 - File Created
-- Additional Comments: 
--
----------------------------------------------------------------------------------
library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;

-- Uncomment the following library declaration if using
-- arithmetic functions with Signed or Unsigned values
--use IEEE.NUMERIC_STD.ALL;

-- Uncomment the following library declaration if instantiating
-- any Xilinx primitives in this code.
--library UNISIM;
--use UNISIM.VComponents.all;

entity AD2SErrorHandling is
    Port (FilterClk: in STD_LOGIC;   --Clock source for this component. Must be 1MHz / 1024 = 976.5625Hz, as timings are derived from it.
          SyncLOT: in STD_LOGIC;     --Synchronized version of LOT signal - Loss of track (output from AD2S1200), active low
          SyncDOS: in STD_LOGIC;     --Synchronized version of DOS signal - Degradation of signal (output from AD2S1200), active low
          FilteredLOT: out STD_LOGIC;--Persisted state of LOT, active high as a status indicator
          FilteredDOS: out STD_LOGIC;--Persisted state of DOS, active high as a status indicator
          LOS: out STD_LOGIC         --Loss of signal (combination of LOT and DOS, when both are active), active high as a status indicator
    );
end AD2SErrorHandling;

architecture Behavioral of AD2SErrorHandling is

  constant CZeroCounter: STD_LOGIC_VECTOR(10 downto 0) := CONV_STD_LOGIC_VECTOR(2000, 11);
  constant COneCounter: STD_LOGIC_VECTOR(9 downto 0) := CONV_STD_LOGIC_VECTOR(53, 10);
  constant CActiveLimit: STD_LOGIC_VECTOR(9 downto 0) := CONV_STD_LOGIC_VECTOR(3, 10); --if a counter is greater than this, then the filtered signal is active
  
  signal FilteredLOT_Temp: STD_LOGIC;
  signal FilteredDOS_Temp: STD_LOGIC;
  
  signal LOT_FilterCounter0: STD_LOGIC_VECTOR(10 downto 0) := (others => '0');  --counts when 0
  signal DOS_FilterCounter0: STD_LOGIC_VECTOR(10 downto 0) := (others => '0');  --counts when 0
  signal LOT_FilterCounter1: STD_LOGIC_VECTOR(9 downto 0) := (others => '0');  --counts when 1
  signal DOS_FilterCounter1: STD_LOGIC_VECTOR(9 downto 0) := (others => '0');  --counts when 1
  
  signal LOT_Rev: STD_LOGIC; --active 1
  signal DOS_Rev: STD_LOGIC; --active 1
begin

  LOT_Rev <= not SyncLOT;
  DOS_Rev <= not SyncDOS;
    
  process(FilterClk)
  begin
    if FilterClk'event and FilterClk = '1' then
      if LOT_Rev = '1' then
        if LOT_FilterCounter1 < COneCounter then
          LOT_FilterCounter1 <= LOT_FilterCounter1 + 1;
        else
          LOT_FilterCounter0 <= (others => '0'); --reset the '0' counter if the '1' counter reached the limit
        end if;
      else
        if LOT_FilterCounter0 < CZeroCounter then
          LOT_FilterCounter0 <= LOT_FilterCounter0 + 1;
        else
          LOT_FilterCounter1 <= (others => '0'); --reset the '1' counter if the '0' counter reached the limit
          LOT_FilterCounter0 <= (others => '0');
        end if;
      end if;
      
      if DOS_Rev = '1' then
        if DOS_FilterCounter1 < COneCounter then
          DOS_FilterCounter1 <= DOS_FilterCounter1 + 1;
        else
          DOS_FilterCounter0 <= (others => '0'); --reset the '0' counter if the '1' counter reached the limit
        end if;
      else
        if DOS_FilterCounter0 < CZeroCounter then
          DOS_FilterCounter0 <= DOS_FilterCounter0 + 1;
        else
          DOS_FilterCounter1 <= (others => '0'); --reset the '1' counter if the '0' counter reached the limit
          DOS_FilterCounter0 <= (others => '0');
        end if;
      end if;
    end if;
  end process;
  

  LOS <= FilteredLOT_Temp and FilteredDOS_Temp;
  FilteredLOT <= FilteredLOT_Temp;
  FilteredDOS <= FilteredDOS_Temp;


  process(FilterClk)
  begin
    if FilterClk'event and FilterClk = '1' then
      if LOT_FilterCounter1 > CActiveLimit then
        FilteredLOT_Temp <= '1';
      else
        FilteredLOT_Temp <= '0';
      end if;
      
      if DOS_FilterCounter1 > CActiveLimit then
        FilteredDOS_Temp <= '1';
      else
        FilteredDOS_Temp <= '0';
      end if;
    end if;
  end process;
   
end Behavioral;

